--[[
Name: Legion Bar Keeper
Project Revision: @project-revision@
File Revision: @file-revision@
Author(s): Netrox
Website: http://wow.curseforge.com/projects/legion-bar-keeper/
Git Repo: git://git.curseforge.com/wow/legion-bar-keeper/mainline.git
License: All rights reserved.
]]

LBK = CreateFrame('frame')
local LBK = LBK

local SPEC_CHANGE_SPELL = 200749
LBK.SUMMON_RANDOM_FAVORITE_MOUNT_ID = 268435455
LBK.SUMMON_RANDOM_FAVORITE_MOUNT_SPELL = 150544

LBK:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
LBK:RegisterEvent("ADDON_LOADED")

function LBK:ADDON_LOADED(event, addon)
    if addon == "LegionBarKeeper" then

        -- Revision/Version
        if "0.0.1_BETA" ~= "@project" .. "-version@" then
            self.rev = "0.0.1_BETA"
        else
            self.rev = "Git"
        end

        self:UnregisterEvent("ADDON_LOADED")
        self.ADDON_LOADED = nil
        if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
    end
end

function LBK:Print(t)
    DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Legion Bar Keeper|r: " .. tostring(t))
end

function LBK:PLAYER_LOGIN(event, addon)
    local defaults = {
        profile = {
            bars = {
                ["**"] = {
                    name = "",
                    buttons = {
                        ["**"] = {
                            spec1 = "",
                            spec2 = "",
                            spec3 = "",
                            spec4 = "",
                            override = false,
                            target = "",
                        }
                    }
                }
            }
        }
    }
    self.db = LibStub("AceDB-3.0"):New("LegionBarKeeperDB", defaults)

    self:RegisterEvent("UNIT_SPELLCAST_STOP")
    self:RegisterEvent("UNIT_SPELLCAST_START")
    self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
    self:RegisterEvent("ACTIONBAR_SLOT_CHANGED")

    self.changingSpec = false

    SLASH_LegionBarKeeper1 = "/legionbarkeeper";
    SLASH_LegionBarKeeper2 = "/lbk";
    SlashCmdList["LegionBarKeeper"] = function()
        self:OpenOptions()
    end
end

function LBK:OpenOptions()
    if not self.optionsBuild then self:BuildOptions() end
    local width = 600 + (GetNumSpecializations() * 170)
    LibStub("AceConfigDialog-3.0"):SetDefaultSize("Legion Bar Keeper", width, 850)
    LibStub("AceConfigDialog-3.0"):Open("Legion Bar Keeper")
end

function LBK:UNIT_SPELLCAST_START(event, unit, _, _, _, spellID)
    if "player" == unit and spellID == SPEC_CHANGE_SPELL then
        -- spec change started
        self.changingSpec = true
    end
end

function LBK:UNIT_SPELLCAST_STOP(event, unit, _, _, _, spellID)
    if "player" == unit and spellID == SPEC_CHANGE_SPELL then
        -- spec change cancelled
        self.changingSpec = false
    end
end

function LBK:UNIT_SPELLCAST_SUCCEEDED(event, unit, _, _, _, spellID)
    if "player" == unit and spellID == SPEC_CHANGE_SPELL then
        -- spec has been changed
        self.changingSpec = false

        for bar = 1, 12 do
            local offset = (bar - 1) * 12
            for btn = 1, 12 do
                self:SetActionSlots(bar)
                self:CheckActionSlot(bar, btn, offset + btn, false, false)
            end
        end
        LibStub("AceConfigRegistry-3.0"):NotifyChange("Legion Bar Keeper")
    end
end

function LBK:ACTIONBAR_SLOT_CHANGED(event, changedSlot)
    if self.changingSpec then
        -- ignore action slot events during spec change
        return
    end

    if changedSlot == 0 then
        --[===[@debug@
        -- LBK:Print("All Buttons have changed, refresh?!")
        --@end-debug@]===]

        -- indicates that all slots have been updated
        -- this should only happen on spec change
        -- (handled in UNIT_SPELLCAST_SUCCEEDED)
        -- and after a loading screen (ignored)
        return
    end

    local bar = math.ceil(changedSlot / 12, 0);
    local slot = changedSlot - ((bar - 1) * 12)
    self:CheckActionSlot(bar, slot, changedSlot, true, true)
end

local options = {
    type = "group",
    childGroups = "tab",
    get = function(k)
        return ""
    end,
    set = function(k, v) end,
    args = {},
}

function LBK:CheckActionSlot(bar, slot, position, triggerEvent, setTargetSlot)
    local buttonAction
    if HasAction(position) then
        local actionType, actionData = GetActionInfo(position);
        buttonAction = actionType .. ":" .. actionData
    end

    local configChanged = false
    if setTargetSlot and self.db.profile['bars'][bar]['buttons'][slot]["override"] then
        self.db.profile['bars'][bar]['buttons'][slot]["target"] = buttonAction
        configChanged = true
    end
    if self.db.profile['bars'][bar]['buttons'][slot]["spec" .. GetSpecialization()] ~= buttonAction then
        self.db.profile['bars'][bar]['buttons'][slot]["spec" .. GetSpecialization()] = buttonAction
        configChanged = true
    end
    if configChanged and triggerEvent then
        LibStub("AceConfigRegistry-3.0"):NotifyChange("Legion Bar Keeper")
    end
end

function LBK:SetActionSlots(bar)
    local offset = (bar - 1) * 12
    for btn = 1, 12 do
        local db = self.db.profile['bars'][bar]['buttons'][btn]
        if db["override"] then
            if db["target"] then
                -- set action button
                local value = db["target"];
                local actionType, actionData = self.ParseValue(nil, value);
                self.Pickup(actionType, actionData)
                PlaceAction(btn + offset)
                ClearCursor()
            else
                -- clear action button
                PickupAction(btn + offset)
                ClearCursor()
            end
        end
    end
end

function LBK:ClearMountJournalSettings()
    -- save settings
    local oldSearchText = MountJournal.searchBox:GetText();
    local oldCollected = C_MountJournal.GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED);
    local oldNotCollected = C_MountJournal.GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_NOT_COLLECTED);

    local oldSources = {};
    local numSources = C_PetJournal.GetNumPetSources();
    for i = 1, numSources do
        oldSources[i] = C_MountJournal.IsSourceChecked(i)
    end
    -- save settings

    -- enable all filter
    C_MountJournal.SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED, true);
    C_MountJournal.SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_NOT_COLLECTED, true);
    C_MountJournal.SetAllSourceFilters(true);
    -- enable all filter

    return oldSearchText, oldCollected, oldNotCollected, oldSources
end

function LBK:RestoreMountJournalSettings(oldSearchText, oldCollected, oldNotCollected, oldSources)
    -- restore settings
    MountJournal.searchBox:SetText(oldSearchText);
    C_MountJournal.SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED, oldCollected);
    C_MountJournal.SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_NOT_COLLECTED, oldNotCollected);

    local numSources = C_PetJournal.GetNumPetSources();
    for i = 1, numSources do
        C_MountJournal.SetSourceFilter(i, oldSources[i]);
    end
    -- restore settings
end

function LBK:FindMountIdBySpellId(spellId)
    local oldSearchText, oldCollected, oldNotCollected, oldSources = self:ClearMountJournalSettings()
    -- ensure we have a number
    spellId = tonumber(spellId)

    local mountIndex = -1
    local Num = C_MountJournal.GetNumMounts();
    if (spellId == LBK.SUMMON_RANDOM_FAVORITE_MOUNT_SPELL) then
        mountIndex = 0;
    end
    for i = 1, Num do
        if (select(2, C_MountJournal.GetDisplayedMountInfo(i)) == spellId) then
            mountIndex = select(12, C_MountJournal.GetDisplayedMountInfo(i));
        end
    end

    self:RestoreMountJournalSettings(oldSearchText, oldCollected, oldNotCollected, oldSources)

    return mountIndex
end

function LBK:FindMountIndexByMountId(mountId)
    local oldSearchText, oldCollected, oldNotCollected, oldSources = self:ClearMountJournalSettings()

    -- ensure we have a number
    mountId = tonumber(mountId)

    local mountIndex = -1
    local Num = C_MountJournal.GetNumMounts();
    if (mountId == LBK.SUMMON_RANDOM_FAVORITE_MOUNT_ID) then
        mountIndex = 0;
    end
    for i = 1, Num do
        if (select(12, C_MountJournal.GetDisplayedMountInfo(i)) == mountId) then
            mountIndex = i;
        end
    end

    self:RestoreMountJournalSettings(oldSearchText, oldCollected, oldNotCollected, oldSources)

    return mountIndex
end

function LBK:BuildOptions()
    self.optionsBuild = true
    self.options = options
    options.name = "Legion Bar Keeper " .. CDB.rev

    LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Legion Bar Keeper", options)

    self.options.args.bars = self:createGroup("Settings")

    local db = self.db.profile
    self.options.args.bars.get = function(key)
        if key.arg.button then
            return db['bars'][key.arg.bar]['buttons'][key.arg.button][key.arg.arg]
        else
            return db['bars'][key.arg.bar][key.arg.arg]
        end
    end
    self.options.args.bars.set = function(key, value)
        if key.arg.button then
            db['bars'][key.arg.bar]['buttons'][key.arg.button][key.arg.arg] = value
        else
            db['bars'][key.arg.bar][key.arg.arg] = value
        end
    end

    local bars = self.options.args.bars.args

    bars.load = self:createExecute("Refresh current spec", function()
        for bar = 1, 10 do
            local offset = (bar - 1) * 12
            for btn = 1, 12 do
                LBK:CheckActionSlot(bar, btn, offset + btn, false, false)
            end
            -- update gui
            LibStub("AceConfigRegistry-3.0"):NotifyChange("Legion Bar Keeper")
        end
    end)

    local numberOfSpecs = GetNumSpecializations()
    for bar = 1, 10 do
        bars["bar" .. bar] = self:createGroup(function()

            local syncedSlots = 0
            for btn = 1, 12 do
                if db['bars'][bar]['buttons'][btn]["override"] == true then
                    syncedSlots = syncedSlots + 1
                end
            end

            local name = "Bar " .. bar;
            if db['bars'][bar]["name"] ~= "" then
                name = db['bars'][bar]["name"]
            end

            return name .. " (" .. syncedSlots .. " slots synced)"
        end)

        local buttons = bars["bar" .. bar].args;

        buttons.name = self:createInput("Name", { bar = bar, arg = "name" })
        buttons.name.width = "double"
        buttons.nameSpacer = self:createDescription("")
        buttons.nameSpacer.width = "full"

        for spec = 1, numberOfSpecs do
            local _, name = GetSpecializationInfo(spec)
            buttons["spec" .. spec .. "header"] = self:createDescription(name)
        end
        buttons.overrideHeader = self:createDescription("Override")
        buttons.targetAction = self:createDescription("with Action")

        for button = 1, 12 do

            local argBuilder = function(arg)
                return {
                    bar = bar,
                    button = button,
                    arg = arg,
                }
            end

            local buttonKey = "bar" .. bar .. "btn" .. button
            for spec = 1, numberOfSpecs do
                buttons[buttonKey .. "spec" .. spec] = self:ActionSlotWidget(bar, button, spec, argBuilder("spec" .. spec))
            end
            buttons[buttonKey .. "override"] = self:createToggle("Enable", argBuilder("override"))
            buttons[buttonKey .. "target"] = self:ActionSlotWidget(bar, button, 0, argBuilder("target"))
        end

        for i = 1, numberOfSpecs + 1 do
            buttons["spacer" .. i] = self:createDescription("")
        end
        buttons.save = self:createExecute("Set for current spec", function()
            LBK:SetActionSlots(bar)
        end)
    end
    options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
end


-------------------------------------
-------------------------------------
---------- Helper  Methods ----------
-------------------------------------
-------------------------------------

local optionOrder = 0
local function getOrder()
    optionOrder = optionOrder + 1
    return optionOrder
end

function LBK:createGroup(name)
    return {
        type = "group",
        name = name,
        order = getOrder(),
        args = {},
    }
end

function LBK:ActionSlotWidget(bar, button, spec, arg)
    return {
        name = "",
        order = getOrder(),
        control = 'ActionSlot',
        disabled = function() return spec ~= 0
        end,
        type = 'input',
        arg = arg;
    }
end

function LBK:createDescription(name)
    return {
        name = name,
        type = "description",
        fontSize = "large",
        width = "normal",
        order = getOrder(),
    }
end

function LBK:createToggle(name, arg)
    return {
        type = "toggle",
        name = name,
        order = getOrder(),
        arg = arg,
    }
end

function LBK:createExecute(name, func)
    return {
        type = "execute",
        name = name,
        order = getOrder(),
        func = func,
    }
end

function LBK:createInput(name, arg)
    return {
        type = "input",
        name = name,
        order = getOrder(),
        arg = arg,
    }
end
